
#ifndef RTCPPACKET_HXX
#define RTCPPACKET_HXX

/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */

static const char* const RtcpPacket_hxx_Version =
    "$Id: RtcpPacket.hxx,v 1.10 2001/08/02 00:03:17 icahoon Exp $";


#include "Rtcp.hxx"
#include "rtpTypes.h"


/// Compound RTCP control packet
class RtcpPacket
{
    public:
        ///
        RtcpPacket ();
        ///
        ~RtcpPacket ();

        /// Pointer to beginning of memory of packet
        char* getPacketData ();

        /** Pointer to beginning of free mememory of packet.
            Must call allocData() afterwards saying how much you used
         **/
        char* freeData ();

        /// Increases packet memory usage by newSize. Returns newSize
        int allocData (int newSize);

        /// Total mem allocated in packet
        int getPacketAlloc();

        /// Unused memory in payload area
        int getUnused ();

        /** Sets total unused memory in packet.
            Useful when receiving packet into buffer
         **/
        void setTotalUsage (int size);

        /** Total unused memory in packet.
            Useful when transmitting packet
         **/
        int getTotalUsage ();

        ///
        int getVersion ();
        ///
        int getPadbyteFlag ();
        ///
        int getCount ();
        ///
        RtcpType getPayloadType ();
        ///
        int getLength ();

        /// debug
        void printPacket ();

    private:
        /// Default RTCP packet size
        static const int PACKETSIZE;

        /// Pointer to raw packet memory
        char* packetData;

        /// Allocated raw packet memory size
        int packetAlloc;

        /// Amount of unused packet memory
        int unusedSize;
};


#endif // RTCPPACKET_HXX
