#if !defined(VOCAL_EVENT_OBSERVER_HXX)
#define VOCAL_EVENT_OBSERVER_HXX

/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */


static const char* const EventObserver_hxx_Version = 
    "$Id: EventObserver.hxx,v 1.4 2002/04/24 01:54:31 bko Exp $";


#include "global.h"
#include "Observer.hxx"
#include "EventSubject.hxx"


/** Infrastructure common to VOCAL.
 */
namespace Vocal 
{


/** Infrastructure in VOCAL related to "algorithms and the assignment of
 *  responsibilities between objects".<br><br>
 *
 *  See Design Patterns, ISBN 0-201-63361-2, Chapter 5.
 */
namespace Behavioral
{


template <class> class EventSubject;


/** Specialized observer to handle events.<br><br>
 *
 *  @see    Vocal::Behavioral::Observer
 *  @see    Vocal::Behavioral::EventSubject
 */
template<class Event>
class EventObserver : public Vocal::Behavioral::Observer
{
    protected:

    	/** This object will only respond to events from the given
	 *  subject.
	 */
    	EventObserver(EventSubject<Event> &);

    public:

    	/** Virtual destructor.
    	 */
	virtual ~EventObserver();
	

    	/** When update() is called by the given subject, the callback
	 *  interestedIn() is called, and on success, onEvent() is then
	 *  called, allowing the extending class to process the event.
    	 */
    	void	    	update(Subject &);

    protected:
    
    	/** This callback must return true to indicate that the extending
	 *  wishes to continue processing this event.
    	 */
    	virtual bool	interestedIn(const Event &) = 0;


    	/** This callback allows the extending class to process the incoming 
	 *  event.
    	 */
    	virtual void	onEvent(Event) = 0;
	
    private:
    
    	EventSubject<Event>   	&   myEventSubject;
};


#include "EventObserver.cc"


} // namespace Behavioral
} // namespace Vovida


#endif // !defined(VOCAL_EVENT_OBSERVER_HXX)
