#ifndef VFILESYSTEM_HXX
#define VFILESYSTEM_HXX

/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */





static const char* const VFileSystemVersion =
    "$Id: VFileSystem.hxx,v 1.1 2001/03/30 02:49:21 icahoon Exp $";



#include <list>
#include <string>

#include "VException.hxx"

/// List of strings
typedef list < string > StringList;

/// TimeStame used for setting file modification data
typedef unsigned long TimeStamp;


/// Class to abstact file system operations
class VFileSystem
{
    public:
        /// read in a file
        static string
        readFile( const string& fileName )
        throw(VException&);

        /// Write data to the given file
        static void
        VFileSystem::writeFile( const string& fileName, const string& data )
        throw(VException&);

        /// Set the time stamp on a file
        static void
        setFileTime( const string& fileName, TimeStamp time )
        throw(VException&);

        /// Get the time stamp of when a file was last modified
        static TimeStamp
        getFileTime( const string& fileName)
        throw(VException&);

        /// Get the size of a file
        static int
        getFileSize( const string& fileName)
        throw(VException&);

        /// Remove a file
        static void
        removeFile( const string& fileName )
        throw(VException&);

        /// Check if a file exists
        static bool
        fileExists( const string& fileName )
        throw(VException&);

        /// Create a directory
        static void
        createDir( const string& dirName )
        throw(VException&);

        /// Remove a directory
        static void
        removeDir( const string& dirName )
        throw(VException&);

        /// Check if a directory exists
        static bool
        dirExists( const string& dirName )
        throw(VException&);

        /// Find all the entires in a dirctory (ignores . and ..)
        static StringList
        readDir( const string& dirName)
        throw(VException&);
};

#endif
