/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */


static const char* const ControlEvent_cxx_Version =
    "$Id: ControlEvent.cxx,v 1.6 2001/10/26 21:47:52 icahoon Exp $";


#include "global.h"
#include "ControlEvent.hxx"


using namespace Vocal;
using Vocal::Services::ControlEvent;
using Vocal::Services::Event;
using Vocal::Threads::Mutex;

Mutex	ControlEvent::myNextIdMutex;


int ControlEvent::myNextId = 0;


ControlEvent::ControlEvent(
    Type pType,
    FifoBase < Sptr < Event > > * pFifo
)
    :	myType(pType),
    	myReturnCode(-1),
    	myFifo(pFifo),
    	myId(0)
{
    myNextIdMutex.lock();
    myId = ++myNextId;
    myNextIdMutex.unlock();
}


ControlEvent::ControlEvent(
    Type    	    	    	    pType,
    FifoBase < Sptr < Event > > &   pFifo
)
    :	myType(pType),
    	myReturnCode(-1),
    	myFifo(&pFifo),
    	myId(0)
{
    myNextIdMutex.lock();
    myId = ++myNextId;
    myNextIdMutex.unlock();
}


ControlEvent::~ControlEvent()
{
}



const char* const
ControlEvent::name() const
{
    return ( "ControlEvent" );
}


ControlEvent::Type
ControlEvent::getType() const
{
    return ( myType );
}


void
ControlEvent::setReturnCode(int rc)
{
    myReturnCode = rc;
}


int
ControlEvent::getReturnCode() const
{
    return ( myReturnCode );
}


bool    
ControlEvent::isRequest() const
{
    return ( myType <= MAX_REQUEST );
}


bool    
ControlEvent::isResponse() const
{
    return ( MIN_RESPONSE <= myType && myType <= MAX_RESPONSE );
}


void    
ControlEvent::response()
{
    if ( isRequest() )
    {
        // *sigh*
        //
        myType = static_cast<Type>(
                    static_cast<int>(myType) + static_cast<int>(MIN_RESPONSE)
                );
    }
    assert( isResponse() );
}
        

void
ControlEvent::setReplyFifo(
    FifoBase< Sptr<Event> > & fifo
)
{
    myFifo = &fifo;
}


FifoBase< Sptr<Event> > *
ControlEvent::getReplyFifo() const
{
    return ( myFifo );
}


int
ControlEvent::getControlEventId() const
{
    return ( myId );
}


ostream &
ControlEvent::writeTo(ostream & out) const
{
    size_t  type = static_cast<size_t>(myType);
    size_t  nameIndex = ( type < TOTAL_TYPES ? type : TOTAL_TYPES );

    char *	controlEventName[TOTAL_TYPES+1] =
    {
    	"Start",
        "Stop",
        "Shutdown",
        "Start response",
        "Stop response",
        "Shutdown response",
        "Unknown"
    };

    return ( out << name() << ": " << controlEventName[nameIndex] );
}
