#if !defined(VOCAL_MT_SERVICE_MANAGER_HXX)
#define VOCAL_MT_SERVICE_MANAGER_HXX

/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */


static const char* const MTMTServiceManager_hxx_Version = 
    "$Id: MTServiceManager.hxx,v 1.1 2001/10/26 21:47:52 icahoon Exp $";


#include "ServiceManager.hxx"
#include "FifoBase.h"
#include "Event.hxx"
#include "Sptr.hxx"
#include <list>


namespace Vocal {
    namespace Threads {
        class Thread;
    }
    namespace Signals {
        class SignalAction;
    }
}
    

/** Infrastructure common to VOCAL.
 */
namespace Vocal 
{


/** Infrastructure common to VOCAL to provide event based services.<br><br>
 */
namespace Services
{


using Vocal::Threads::Thread;
using Vocal::Signals::SignalAction;


class Service;


/** 
 */
class MTServiceManager : public Vocal::Services::ServiceManager
{

    public:

    	/** Create the service manager, given the fifo that the
         *  will use.
    	 */    
    	MTServiceManager(
	    FifoBase< Sptr<Event> > 	&   serviceMgrFifo,
            SignalAction                *   signalAction = 0
	);
	

    	/** Virtual destructor
    	 */    
	virtual ~MTServiceManager();


    	/** Request the service manager to manage the given service.
	 */	
	void	manage(Service &);


    	/** Request the service manager to unmanage all the services.
	 */	
	void	unmanage();

	/** Starts all managed services. They are started serially, in the 
         *  order they were added to the manager.
	 */
	virtual ReturnCode	start();


    	/** Blocks waiting for either a signal, or an error message from
         *  a service indicating the process needs to terminate.
	 */
	virtual ReturnCode	run();


	/** Stops all managed services. They are stopped serially, in the
         *  reverse order as from the order they were started.
	 */
	virtual	void	 	stop();


	/** Shuts down all managed services. They are shutdown serially, in
         *  the reverse order from the order they were started.
	 */
	virtual	void	    	shutdown();


    private:

	void	    	        blockSignals();
	void	    	        unblockSignals();
	bool  	                signalCaught();
	
    	bool  	    	        block();
	
        struct ServiceInfo
        {
            Service     *   service;
            Thread      *   thread;
        };

    	typedef	    list<ServiceInfo> 	ServiceList;

	ServiceList 	    	    	myServiceList;
	FifoBase< Sptr<Event> >     &	myFifo;
};


} // namespace Services
} // namespace Vocal


#endif // !defined(VOCAL_MT_SERVICE_MANAGER_HXX)
