#if !defined(VOCAL_POLL_FIFO_SERVICE_HXX)
#define VOCAL_POLL_FIFO_SERVICE_HXX

/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */


static const char* const PollFifoService_hxx_Version = 
    "$Id: PollFifoService.hxx,v 1.3 2001/10/26 21:47:52 icahoon Exp $";


#include "Service.hxx"
#include "PollFifo.hxx"
#include <string>


namespace Vocal {
    namespace Transport {
    	class Protocol;
	class ProtocolException;
    }
}


/** Infrastructure common to VOCAL.
 */
namespace Vocal 
{


/** Infrastructure common to VOCAL to provide event based services.<br><br>
 */
namespace Services
{


/** Extends the Service class, using a PollFifo.<br><br>
 *
 *  @see    Vocal::Services::Service
 *  @see    Vocal::PollFifo
 *  @see    Vocal::Transport::Protocol
 *  @see    Vocal::SystemException
 *  @see    Vocal::Transport::ProtocolException
 *  @see    Vocal::Services::Event
 */
class PollFifoService : public Vocal::Services::Service
{

    public:

    	
	/** Create the PollFifoServer an optional name.
	 */
    	PollFifoService(const char * name = 0);


    	/** Virtual destructor
	 */
	virtual     	    	~PollFifoService();


    	/** Accessor for PollFifo.
	 */
	FifoBase< Sptr<Event> >     	&   getFifo();


    	/** Accessor for PollFifo.
	 */
	const FifoBase< Sptr<Event> >	&   getFifo() const;


    	/** Register the protocol for activity detection.
	 */	
	void	    	registerProtocol(Protocol &);


    	/** Update the protocol's activity detection.
    	 */	
    	void	    	updateProtocol(Protocol &);


    	/** Register the protocol for activity detection.
	 */	
	void	    	unregisterProtocol(Protocol &);


    protected:

    	/** Handle an exception thrown by the fifo during wait. 
    	 *  The default behavior is to ignore the exception.
    	 *  Return true to shutdown, which will cause this thread to exit.
	 */
	virtual	void	onVException(VException &);


    	/** Handle a transport exception thrown by the fifo during
    	 *  wait. If we get this, poll is probably broken and other 
    	 *  threads probably can't send events to us, so we need
    	 *  to do a hard shutdown. Return true to shutdown, which
    	 *  will cause this thread to exit.
	 */
	virtual	void	onSystemException(SystemException &);


    	/** Handle a protocol exception thrown directly by a protocol
    	 *  or thrown indirectly by a MessageObserver using a
    	 *  protocol. The default action is to do nothing. Return true 
    	 *  to shutdown, which will cause this thread to exit.
	 */	
	virtual	void	onProtocolException(Transport::ProtocolException &);
		

	/** Calls the FifoBase's processMessages, then processes
    	 *  the protocols.
	 */
    	bool	    	processMessages(int numberFdsActive, ReturnCode &);
		

    private:

    	void	    	processProtocols(int numberFdsActive);

        PollFifo< Sptr<Event> >	    	myFifo;
};


} // namespace Services
} // namespace Vocal


#endif // !defined(VOCAL_POLL_FIFO_SERVICE_HXX)
