#if !defined(VOCAL_PROTOCOL_SERVICE_HXX)
#define VOCAL_PROTOCOL_SERVICE_HXX

/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF ,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */


static const char* const ProtocolService_hxx_Version = 
    "$Id: ProtocolService.hxx,v 1.2 2001/10/26 21:47:52 icahoon Exp $";


#include "PollFifoService.hxx"
#include "PollFifo.hxx"
#include "Protocol.hxx"
#include "Sptr.hxx"
#include "Event.hxx"
#include <list>


namespace Vocal {
    namespace Configuration {
        class Config;
    }
}


/** Vovida Open Communication Application Library.
 */
namespace Vocal
{


/** 
 */
namespace Services
{


using Vocal::Services::PollFifoService;
using Vocal::Services::Event;
using Vocal::Configuration::Config;
using Vocal::Transport::Protocol;
using Vocal::ReturnCode;
using std::list;


/** 
 */
class ProtocolService : public PollFifoService
{
    public:


	/** 
	 */
    	ProtocolService(
	    const Config	    &	config,
            const char              *   name
	);


	/** Virtual destuctor.
	 */
	virtual ~ProtocolService();


    protected:
    
    	/** Starts the service.
	 */
	ReturnCode  onStart();


    	/** Stops the service. The service may be started again.
	 */
	ReturnCode  onStop();


    	/** Shutdowns the service. The service will exit.
	 */
	ReturnCode  onShutdown();


        /**
         */
        virtual void    createProtocols();    
        

    	/** Clean up allocated memory.
    	 */
    	virtual void    clear();


	/** 
    	 */
    	list< Sptr<Protocol> >     myProtocols;


    	/** The configuration information for the application.
	 */
	const Config    	&   myConfig;
};


} // namespace Services
} // namespace Vocal


#endif // !defined(VOCAL_PROTOCOL_SERVICE_HXX)
