#if !defined(PERFORMANCE_DB_DOT_HXX)
#define PERFORMANCE_DB_DOT_HXX

/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */


static const char* const PerformaceDb_hxx_Version = 
    "$Id: PerformanceDb.hxx,v 1.2 2001/02/02 21:36:15 icahoon Exp $";


#include "StatisticsDb.hxx"


/** Infrastructure common to VOCAL.<br><br>
 */
namespace Vocal 
{


/** Infrastructure in VOCAL to measuring, recording and reporting
 *  statistics.<br><br>
 */
namespace Statistics
{


class PerfDuration;
class PerfTotal;


/** Database dedicated to gather performance statistics.<br><br>
 */
class PerformanceDb : public Vocal::Statistics::StatisticsDb
{
    protected:


	/** Create the database.<br><br> Only used by instance().
	 */
	PerformanceDb();


    public:
    

	/** Create the singleton database.<br><br>
	 */
    	static PerformanceDb &	instance();


	virtual ~PerformanceDb();


	/** Finds the combined PerfDuration statistic in the database
         *  that matches the type (db key) of the passed in statistic. 
         *
         *  Returns 0 if not found.
	 */
	virtual PerfDuration * find(const PerfDuration &) const;


	/** Finds the combined PerfTotal statistic in the database
         *  that matches the type (db key) of the passed in statistic. 
         *
         *  Returns 0 if not found.
	 */
	virtual PerfTotal * find(const PerfTotal &) const;


    private:


    	static PerformanceDb *	m_instance;
};


} // namespace Statistics
} // namespace Vocal 


#endif // !defined(PERFORMANCE_DB_DOT_HXX)
