/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */


static const char* const RWMutex_cxx_Version =
    "$Id: RWMutex.cxx,v 1.2 2001/08/10 04:02:08 icahoon Exp $";


#include "global.h"
#include "RWMutex.hxx"
#include "Lock.hxx"
#include <cassert>


using Vocal::Threads::RWMutex;
using Vocal::Threads::Lock;


RWMutex::RWMutex()
    :   myReaderCount(0),
    	myWriterHasLock(false),
	myPendingWriterCount(0)
{
}


RWMutex::~RWMutex()
{
}


void
RWMutex::readlock()
{
    Lock    lock(myMutex);
    
    while ( myWriterHasLock || myPendingWriterCount > 0 )
    {
    	myReadCondition.wait(&myMutex);
    }

    myReaderCount++;
}


void
RWMutex::writelock()
{
    Lock    lock(myMutex);

    myPendingWriterCount++;
    
    while ( myWriterHasLock || myReaderCount > 0 )
    {
    	myPendingWriteCondition.wait(&myMutex);
    }

    myPendingWriterCount--;
    
    myWriterHasLock = true;
}


void
RWMutex::lock()
{
    writelock();
}


void
RWMutex::unlock()
{
    Lock    lock(myMutex);
    
    // Unlocking a write lock.
    //
    if ( myWriterHasLock )
    {
    	assert( myReaderCount == 0 );
	
    	myWriterHasLock = false;

        // Pending writers have priority. Could potentially starve readers.
        //
    	if ( myPendingWriterCount > 0 )
	{	
	    myPendingWriteCondition.signal();
	}

        // No writer, no pending writers, so all the readers can go.
        //	
	else
	{
	    myReadCondition.broadcast();
	}

    }
    
    // Unlocking a read lock.
    //
    else
    {
    	assert( myReaderCount > 0 );
	
    	myReaderCount--;
	
	if ( myReaderCount == 0 && myPendingWriterCount > 0 )
	{
	    myPendingWriteCondition.signal();
	}
    }
}


u_int32_t   	
RWMutex::readerCount() const
{
    return ( myReaderCount );
}


u_int32_t   	
RWMutex::pendingWriterCount() const
{
    return ( myPendingWriterCount );
}
