#if !defined(VOCAL_RWMUTEX_HXX)
#define VOCAL_RWMUTEX_HXX

/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */


static const char* const RWMutex_hxx_Version =
    "$Id: RWMutex.hxx,v 1.1 2001/03/23 13:43:10 icahoon Exp $";


#include "Lockable.hxx"
#include "Mutex.hxx"
#include "Condition.hxx"


/** Infrastructure common to VOCAL.
 */
namespace Vocal 
{


/** Infrastructure common to VOCAL to create and manage threads.
 */
namespace Threads
{


/** Read/write mutex.<br><br>
 *
 *  Allows serialized write access and concurrent read access.
 */
class RWMutex : public Lockable
{
    public:

        /** Create a lock.
         */
        RWMutex();


        /** Destroy the lock.
         */        
        virtual ~RWMutex();
        
        
        /** Obtain the read lock. This will block until all writers have
         *  completed.
         */ 
        virtual void    readlock();
        

        /** Obtain the write lock. Provides serialized writing.
         */
        virtual void    writelock();
        

    	/** Defaults to the strictest lock, the writelock.
	 */
        virtual void 	lock();
    

        /** Unlocks either the read or write lock.
         */
        virtual void    unlock();


    	/** Accessor to readerCount.
	 */
    	u_int32_t   	readerCount() const;
	

    	/** Accessor to pendingWriterCount.
	 */
	u_int32_t   	pendingWriterCount() const;


    private:


        /** Access to the lock is synchronous.
         */
        Mutex      myMutex;


        /** Readers will block until all writers have completed.
         */
        Condition  myReadCondition;

        
        /** Pending writers will block while while waiting for the writer
         *  to finish.
         */
        Condition  myPendingWriteCondition;
        

        /** We can have multiple readers blocking.
         */
        u_int32_t   myReaderCount;
        

        /** We can have only a single writer.
         */
        bool        myWriterHasLock;


        /** We can have many pending writers.
         */
        u_int32_t   myPendingWriterCount;
};


} // namespace Threads
} // namespace Vocal


#endif // !defined(VOCAL_RWMUTEX_HXX)
