#ifndef VSEMAPHORE_HXX_
#define VSEMAPHORE_HXX_

/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */

static const char* const VSemaphore_hxx_version =
    "$Id: VSemaphore.hxx,v 1.3 2001/06/29 05:36:29 bko Exp $";

#ifdef WIN32
#include <pthread.h>
#endif

#if defined( __linux__ ) || defined(__FreeBSD__) || defined (WIN32)
#include <semaphore.h>
#else
#include <synch.h>
#endif

#if defined (__linux__) || (__FreeBSD__) || defined (WIN32)
typedef sem_t vsem_t;
#else
typedef sema_t vsem_t;
#endif



/** Class implements a semaphore */
class VSemaphore
{
    public:
	/// default constructor -- sets the count to 1
        VSemaphore();
	/** constructor which can set the count to a value
	    @param available  value to set the count to
	*/
	VSemaphore(int available);
	/// destructor
	virtual ~VSemaphore();

	/** if count == 0, wait on the semaphore.  If count > 0,
	 * decrement count and continue */
        void Wait();
	/// increment count, waking a waiting thread if there are any
        void Post();
	/** increment count by how_many, waking a waiting thread if
	 * there are any */
        void Post(int how_many);
    private:
	vsem_t semaphore;
} ;
#endif
