#ifndef VTHREADPOOL_HXX_
#define VTHREADPOOL_HXX_

/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */

static const char* const VThreadPool_hxx_version =
    "$Id: VThreadPool.hxx,v 1.2 2001/06/29 03:41:33 bko Exp $";

#include <pthread.h>
#include <deque>
#include <list>

#include "VThread.hxx"
#include "VFunctor.hxx"
#include "VThreadInitException.hxx"
#include "Sptr.hxx"

/** this class creates an abstraction for doing work in multiple
 * threads.  VFunctors may be added the work queue.  The waiting
 * threads of the ThreadPool do the work specified by each VFunctor in
 * the work queue.  When the work is done, the thread returns to the
 * thread pool, doing more work if there are any jobs in the work
 * queue or, waiting until there is more work to be done if there are not.
 */

class VThreadPool
{
    public:
        typedef list < Sptr < VThread > > ListOfThreads;

        /// constructor
        VThreadPool(int numThreads = 1);

	/// destructor
        virtual ~VThreadPool();

        /// add a functor to the thread pool to be worked on
        void addFunctor(const VFunctor& functor) throw(VThreadInitException&);

        /** shutdown the threadpool.  if the finish flag is true, wait
	 * till the entire work queue is done
	 */
        void shutdown(bool finish = true);
	
	/// get a list of the worker threads
        const ListOfThreads& getWorkerThreads() const
        {
            return workerThreads_;
        };

        /// call from the thread construction function
        static void* workerProc(void* arg);

    private:
        /// initialize the code
        void init() throw (VThreadInitException&);

        deque < VFunctor > queue_;
        pthread_mutex_t queueLock_;
        pthread_cond_t queueNotEmpty_;
        pthread_cond_t queueNotFull_;
        pthread_cond_t queueEmpty_;
        int numThreads_;
        int queueClosed_;
        int shutdown_;
        int currQueueSize_;

        ListOfThreads workerThreads_;
};

#endif
