/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */

static const char* const vthread_vxworks_cxx_Version =
    "$Id: vthread-vxworks.cxx,v 1.3 2001/08/10 04:02:09 icahoon Exp $";


#ifdef __vxworks

#include "global.h"
#include <cassert>
#include "VThread.hxx"
#include "cpLog.h"

/*********************************************************************
 
VThread for VxWorks
 
*********************************************************************/

VThread::VThread( void )
    : myId(VTHREAD_ID_INVALID)
{
}

VThread::~VThread( void )
{
    if ( taskIdVerify( myId ) == OK )
    {
        // warn user but don't kill thread
        cpLog( LOG_DEBUG, "active VThread:%d going out of scope", myId );
    }
}

int
VThread::spawn( void *( *startFunc )( void* ),
                void *startArgs = 0,
                unsigned long flags = 0,
                unsigned long priority = VTHREAD_PRIORITY_DEFAULT,
                int stackSize = VTHREAD_STACK_SIZE_DEFAULT
              )
{
    if ( myId != VTHREAD_ID_INVALID )
    {
        // make sure VThread doesn't already have an active thread
        // associated with it
        if ( taskIdVerify( myId ) == OK )
        {
            // return without spawning another thread
            cpLog( LOG_ERR, "VThread:%d is already active", myId );
            return VTHREAD_ID_INVALID;
        }
    }

    int vxworksPriority = VTHREAD_PRIORITY_VXWORKS_DEFAULT;
    int vxworksOptions = 0;

    assert( flags == 0 );  // handling tbd

    if ( priority != VTHREAD_PRIORITY_DEFAULT )
    {
        vxworksPriority = priority;
    }

    myId = taskSpawn( 0, vxworksPriority, vxworksOptions,
                      stackSize, (FUNCPTR) startFunc,
                      (int) startArgs, 0, 0, 0, 0, 0, 0, 0, 0, 0 );
    if ( myId <= 0 )
    {
        throw VExceptionMemory(
            "cannot allocate VThread",
            __FILE__,
            __LINE__,
            myId );
    }
    return 0;
}


int
VThread::join( void **status = 0 )
{
    while ( taskIdVerify( myId ) == OK )
    {
        sleep( 1 );
    }
    myId = VTHREAD_ID_INVALID;
    return 0;
}


int
VThread::getPriority( void )
{
    int thePriority = VTHREAD_PRIORITY_DEFAULT;

    if ( taskIdVerify( myId ) == 0 )
    {
        assert( taskPriorityGet( myId, &thePriority ) == OK );
    }
    else
    {
        cpLog( LOG_ERR, "VThread:%d is not active", myId );
    }
    return thePriority;
}


int
VThread::setPriority( int priority )
{
    return ( taskPrioritySet( myId, priority ) );
}


const vthread_t*
VThread::getId( void )
{
    return &myId;
}


const vthread_attr_t*
VThread::getAttributes( void )
{
    return &myAttributes;
}


void
VThread::exit( void )
{
    taskDelete( myId );
    myId = VTHREAD_ID_INVALID;
}


int
sleep( int seconds )
{
    return (int) (taskDelay( seconds * CLOCKS_PER_SEC ) );
}

int
usleep( int useconds )
{
    return (int) (taskDelay( ( useconds * CLOCKS_PER_SEC ) / 1000000 ) );
}

void vusleep(unsigned long usec)
{
    return ( usleep(usec) );
}

const vthread_t
VThread::selfId( void )
{
    return taskIdSelf();
}


#endif
//__vxworks
