#if !defined(VOCAL_VXWORKS_VTHREAD_H)
#define VOCAL_VXWORKS_VTHREAD_H

/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */


static const char* const vxworks_vthread_h_Version =
    "$Id: vthread.h,v 1.1 2001/03/23 13:43:11 icahoon Exp $";


#include "VocalCommon.hxx"
#include <taskLib.h>
#include <sysLib.h>
#include <semLib.h>
#include <cassert>


struct timespec
{
    long    ts_sec;
    long    ts_nsec;
};


typedef int                 vthread_t;
typedef SEM_ID              vmutex_t;
typedef SEM_ID              vcondition_t;


const unsigned long VTHREAD_PRIORITY_VXWORKS_DEFAULT = 150;


inline  
Vocal::ReturnCode
vmutex_init(vmutex_t * mutex)
{
    *mutex = semMCreate(SEM_Q_FIFO);
    assert( *mutex != 0 );

    return ( Vocal::SUCCESS );
}
                

inline
Vocal::ReturnCode
vmutex_destroy(vmutex_t * mutex) 
{
    STATUS errorcode = semDelete(*mutex);
    assert(errorcode == OK);
    
    return ( Vocal::SUCCESS );
}


inline
Vocal::ReturnCode
vmutex_lock(vmutex_t * mutex)
{
    semTake(*mutex, WAIT_FOREVER);
    
    return ( Vocal::SUCCESS );
}


inline
Vocal::ReturnCode
vmutex_unlock(vmutex_t * mutex)
{
    semGive(*mutex);
    
    return ( Vocal::SUCCESS );
}
                

inline
Vocal::ReturnCode
vcond_init(vcond_t * cond)
{
    // Conditional variable is implemented as a binary semaphore
    // which is initialized to empty
    //
    *cond = semBCreate(SEM_Q_FIFO, SEM_EMPTY);
    assert( *cond != 0 );

    return ( Vocal::SUCCESS );
}
                

inline
Vocal::ReturnCode
vcond_destroy(vcond_t * cond)
{
    STATUS errorcode = semDelete(*cond);
    assert(errorcode == OK);

    return ( Vocal::SUCCESS );
}
                

inline
Vocal::ReturnCode
vcond_wait(vcond_t * cond, vmutex_t * mutex)
{
    Vocal::ReturnCode rc = SUCCESS;

    int delayInTicks = WAIT_FOREVER;

    // Note, the call to taskLock() guarantees that the task will not
    // be scheduled out; however, it will still give up the cpu if it
    // blocks on a semaphore, thereby avoiding a deadlock.
    //
    bool taskLockCode = taskLock();
    assert( taskLockCode == OK );

    bool unlockCode = mutex->unlock();
    assert( unlockCode == 0 );

    rc = (int) semTake(*cond, delayInTicks);

    if ( rc != OK )
    {
        assert(errno == S_objLib_OBJ_TIMEOUT);
    }

    bool mutexLockCode = mutex->lock();
    assert( mutexLockCode == 0 );

    bool taskUnlockCode = taskUnlock();
    assert( taskUnlockCode == OK );

    return ( rc );
}


inline
Vocal::ReturnCode
vcond_timedwait(vcond_t * cond, vmutex_t * mutex, timespec * ts)
{
    Vocal::ReturnCode rc = SUCCESS;
    
    int relativeTimeInUs = ts.ts_sec * 1000000 + ts.ts_nsec / 1000;
    assert( relativeTimeInUs > 0 );

    int ticksPerSecond = sysClkRateGet();
    assert( ticksPerSecond > 0 );

    int delayInTicks = relativeTimeInUs * ( ticksPerSecond / 1000000 );
    
    // Note, the call to taskLock() guarantees that the task will not
    // be scheduled out; however, it will still give up the cpu if it
    // blocks on a semaphore, thereby avoiding a deadlock.
    //
    bool taskLockCode = taskLock();
    assert( taskLockCode == OK );

    bool unlockCode = mutex->unlock();
    assert( unlockCode == 0 );

    rc = (int) semTake(*cond, delayInTicks);

    if ( retval != OK )
    {
        assert(errno == S_objLib_OBJ_TIMEOUT);
    }

    bool mutexLockCode = mutex->lock();
    assert( mutexLockCode == 0 );

    bool taskUnlockCode = taskUnlock();
    assert( taskUnlockCode == OK );

    return ( rc );
}


inline
Vocal::ReturnCode
vcond_signal(vcond_t * cond)
{
    return ( (Vocal::ReturnCode) semGive(*cond) );
}


inline
Vocal::ReturnCode
vcond_broadcast(vcond_t * cond)
{
    // TODO
    assert( 0 );
    return ( (Vocal::ReturnCode) semGive(*cond) );
}
                

#endif // !defined(VOCAL_VXWORKS_VTHREAD_H)
